﻿/*
 * By David Barrett, Microsoft Ltd. 2012. Use at your own risk.  No warranties are given.
 * 
 * DISCLAIMER:
 * THIS CODE IS SAMPLE CODE. THESE SAMPLES ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
 * MICROSOFT FURTHER DISCLAIMS ALL IMPLIED WARRANTIES INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR OF FITNESS FOR
 * A PARTICULAR PURPOSE. THE ENTIRE RISK ARISING OUT OF THE USE OR PERFORMANCE OF THE SAMPLES REMAINS WITH YOU. IN NO EVENT SHALL
 * MICROSOFT OR ITS SUPPLIERS BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 * BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING OUT OF THE USE OF OR INABILITY TO USE THE
 * SAMPLES, EVEN IF MICROSOFT HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. BECAUSE SOME STATES DO NOT ALLOW THE EXCLUSION OR LIMITATION
 * OF LIABILITY FOR CONSEQUENTIAL OR INCIDENTAL DAMAGES, THE ABOVE LIMITATION MAY NOT APPLY TO YOU.
 * */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace SOAPe.Encoding
{
    public partial class FormBase64 : Form
    {
        public FormBase64()
        {
            InitializeComponent();
        }

        private System.Text.Encoding SelectedEncoding()
        {
            if (radioButtonEncodingASCII.Checked)
                return System.Text.Encoding.ASCII;

            if (radioButtonEncodingUTF8.Checked)
                return System.Text.Encoding.UTF8;

            return System.Text.Encoding.Unicode;
        }

        private void buttonEncodeText_Click(object sender, EventArgs e)
        {
            try
            {
                byte[] SourceText = SelectedEncoding().GetBytes(textBoxText.Text);
                textBoxBase64.Text = System.Convert.ToBase64String(SourceText);
            }
            catch (Exception ex)
            {
                textBoxBase64.Text = "Error occurred: " + ex.Message;
            }
        }

        private void buttonDecodeToText_Click(object sender, EventArgs e)
        {
            byte[] SourceEncoding=null;
            try
            {
                SourceEncoding = System.Convert.FromBase64String(textBoxBase64.Text);
            }
            catch (Exception ex)
            {
                textBoxText.Text = "Error occurred: " + ex.Message;
                return;
            }
            try
            {
                if (radioButtonEncodingBinary.Checked)
                {
                    // Binary - represent as hex
                    string sHex = "";
                    for (int i = 0; i < SourceEncoding.Length; i++)
                        sHex = sHex + GetHexString(SourceEncoding[i]);
                    textBoxText.Text = sHex;
                }
                else
                    textBoxText.Text = SelectedEncoding().GetString(SourceEncoding);
            }
            catch (Exception ex)
            {
                textBoxText.Text = "Error occurred: " + ex.Message;
            }
        }

        private string GetHexString(byte b)
        {
            Char[] cHexDigits = "0123456789ABCDEFGH".ToCharArray();
            string sHex = "";

            byte lower = (byte)(b & (byte)15);
            byte upper = (byte)((b & (byte)0xF0) >> 4);
            sHex += cHexDigits[upper];
            sHex += cHexDigits[lower];
            return sHex;
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void buttonDecodeToFile_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog oSaveDialog = new SaveFileDialog();
                oSaveDialog.Filter = "Binary files (*.bin)|*.bin|All files (*.*)|*.*";
                oSaveDialog.FilterIndex = 2;
                oSaveDialog.RestoreDirectory = true;

                if (oSaveDialog.ShowDialog() != DialogResult.OK)
                    return;

                Stream oStream = oSaveDialog.OpenFile();
                if (!(oStream == null))
                {
                    byte[] Data = System.Convert.FromBase64String(textBoxBase64.Text);
                    oStream.Write(Data, 0, Data.Length);
                    oStream.Close();
                }
                oSaveDialog.Dispose();
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show("Error occurred: " + Environment.NewLine + ex.Message, "Error", MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
            }
        }

        private void buttonEncodeFile_Click(object sender, EventArgs e)
        {
            try
            {
                OpenFileDialog oOpenDialog = new OpenFileDialog();
                oOpenDialog.Filter = "All files (*.*)|*.*";
                oOpenDialog.FilterIndex = 1;
                oOpenDialog.RestoreDirectory = true;

                if (oOpenDialog.ShowDialog() != DialogResult.OK)
                    return;

                textBoxBase64.Text = "";
                this.Refresh();
                Stream oStream = oOpenDialog.OpenFile();
                byte[] data=new byte[oStream.Length];
                oStream.Read(data, 0, Convert.ToInt32(oStream.Length));
                oStream.Close();
                textBoxBase64.Text = Convert.ToBase64String(data);
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show("Error occurred: " + Environment.NewLine + ex.Message, "Error", MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
            }
        }
    }
}
