﻿/*
 * By David Barrett, Microsoft Ltd. 2012. Use at your own risk.  No warranties are given.
 * 
 * DISCLAIMER:
 * THIS CODE IS SAMPLE CODE. THESE SAMPLES ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
 * MICROSOFT FURTHER DISCLAIMS ALL IMPLIED WARRANTIES INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR OF FITNESS FOR
 * A PARTICULAR PURPOSE. THE ENTIRE RISK ARISING OUT OF THE USE OR PERFORMANCE OF THE SAMPLES REMAINS WITH YOU. IN NO EVENT SHALL
 * MICROSOFT OR ITS SUPPLIERS BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 * BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING OUT OF THE USE OF OR INABILITY TO USE THE
 * SAMPLES, EVEN IF MICROSOFT HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. BECAUSE SOME STATES DO NOT ALLOW THE EXCLUSION OR LIMITATION
 * OF LIABILITY FOR CONSEQUENTIAL OR INCIDENTAL DAMAGES, THE ABOVE LIMITATION MAY NOT APPLY TO YOU.
 * */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Xml;
using System.Windows.Forms;

namespace SOAPe
{
    public partial class FormLogViewer : Form
    {
        private ClassLogger _logger=null;
        private Point _cursorTracking;  // Used when we need to track the mouse pointer position
        private ClassSyntaxHighlighter _syntaxHighlighter;

        public FormLogViewer(ClassLogger Logger)
        {
            InitializeComponent();
            _logger = Logger;
            _logger.LogAdded += _logger_OnLogAdded;
            _syntaxHighlighter = new ClassSyntaxHighlighter(richTextBoxLogView);
            ShowLogIndex();
        }

        ~FormLogViewer()
        {
            try
            {
                _logger.LogAdded -= _logger_OnLogAdded;
            }
            catch { }
        }
        private void ShowLogIndex()
        {
            // Read the logs and update the UI
            listViewLogIndex.Items.Clear();
            richTextBoxLogView.Text = "Please select a log to view.";

            // Check if we have anything to show
            if (_logger.LogHistory == null)
            {
                richTextBoxLogView.Text = "No data found for this session.";
                return;
            }
            if (_logger.LogHistory.Count < 1)
            {
                richTextBoxLogView.Text = "No data found for this session.";
                return;
            }

            // Now read the dates and fill the listbox
            for (int i = 0; i < _logger.LogHistory.Count; i++)
            {
                ListViewItem oItem = new ListViewItem(_logger.LogHistory.Keys.ElementAt(i).ToString("dd MMM yyyy HH:mm:ss"));
                oItem.Tag = _logger.LogHistory.Keys.ElementAt(i);
                string sDetails = _logger.LogHistory[_logger.LogHistory.Keys.ElementAt(i)];
                sDetails = sDetails.Substring(0, sDetails.IndexOf(Environment.NewLine));
                oItem.SubItems.Add(sDetails);
                listViewLogIndex.Items.Add(oItem);
            }
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void _logger_OnLogAdded(object sender, LoggerEventArgs e)
        {
            // New log added
            ListViewItem oItem = new ListViewItem(e.LogTime.ToString("dd MMM yyyy HH:mm:ss"));
            oItem.Tag = e.LogTime;
            string sDetails = e.LogDetails;
            sDetails = sDetails.Substring(0, sDetails.IndexOf(Environment.NewLine));
            oItem.SubItems.Add(sDetails);
            listViewLogIndex.Items.Add(oItem);
        }

        private void listViewLogIndex_SelectedIndexChanged(object sender, EventArgs e)
        {
            richTextBoxLogView.Text = "";
            try
            {
                DateTime oKey = (DateTime)listViewLogIndex.SelectedItems[0].Tag;
                if (_logger.LogHistory.ContainsKey(oKey))
                {
                    string sDetails = _logger.LogHistory[oKey].ToString();
                    sDetails = sDetails.Substring(sDetails.IndexOf(Environment.NewLine)).Trim();
                    richTextBoxLogView.Text = sDetails;
                    SyntaxHighlight();
                }
            }
            catch { }
        }

        private void buttonClearLog_Click(object sender, EventArgs e)
        {
            _logger.ClearHistory();
            ShowLogIndex();
        }

        private void SyntaxHighlight()
        {
            _syntaxHighlighter.SyntaxHighlightIfValidXml(checkBoxIndentXML.Checked);
            richTextBoxLogView.Update();
        }

        private void LogMousePosition(Point CursorPos)
        {
            _cursorTracking = CursorPos;
        }

        private void MoveTextboxCursor(RichTextBox TextBox)
        {
            try
            {
                int iCharPos = TextBox.GetCharIndexFromPosition(_cursorTracking);
                TextBox.SelectionStart = iCharPos;
                TextBox.SelectionLength = 1;
            }
            catch { }
        }

        private void CopyItemId(RichTextBox SourceTextBox, bool RemoveChangeKey = false, bool IDOnly = false)
        {
            // Read the ItemId at the current cursor position and copy to clipboard
            string sText = SourceTextBox.Text;
            int iCaret = SourceTextBox.SelectionStart;
            int iItemIdStart = sText.LastIndexOf("itemid", iCaret, iCaret, System.StringComparison.CurrentCultureIgnoreCase);
            if (iItemIdStart < 0)
            {
                // No ItemId found, let's look for FolderId
                iItemIdStart = sText.LastIndexOf("folderid", iCaret, iCaret, System.StringComparison.CurrentCultureIgnoreCase);
                if (iItemIdStart < 0) return;
            }
            iItemIdStart = sText.LastIndexOf("<", iItemIdStart, iItemIdStart);
            if (iItemIdStart < 0) return;
            int iItemIdEnd = sText.IndexOf("/>", iItemIdStart) + 2;
            if (iItemIdEnd < 0) return;

            string sItemId = sText.Substring(iItemIdStart, iItemIdEnd - iItemIdStart);
            if (!String.IsNullOrEmpty(sItemId))
            {
                SourceTextBox.SelectionStart = iItemIdStart;
                SourceTextBox.SelectionLength = iItemIdEnd - iItemIdStart;
                SourceTextBox.Focus();

                if (IDOnly)
                {
                    // We only want the actual ID (none of the XML)
                    int iIdStart = sItemId.ToLower().IndexOf("id=");
                    if (iIdStart > 0)
                    {
                        iIdStart = sItemId.IndexOf("\"", iIdStart) + 1;
                        int iIdEnd = sItemId.IndexOf("\"", iIdStart);
                        if (iIdEnd > iIdStart)
                            sItemId = sItemId.Substring(iIdStart, iIdEnd - iIdStart);
                    }
                }
                else if (RemoveChangeKey)
                {
                    // Remove the ChangeKey from the ItemId
                    int iChangeKeyStart = sItemId.ToLower().IndexOf("changekey");
                    if (iChangeKeyStart > 0)
                    {
                        int iChangeKeyEnd = sItemId.IndexOf("\"", iChangeKeyStart);
                        iChangeKeyEnd = sItemId.IndexOf("\"", iChangeKeyEnd + 1);
                        if (iChangeKeyEnd > iChangeKeyStart)
                            sItemId = sItemId.Substring(0, iChangeKeyStart) + (sItemId.Substring(iChangeKeyEnd + 1));
                    }
                }
            }

            // Now copy to clipboard
            System.Windows.Forms.Clipboard.SetText(sItemId);
        }
        
        private void copyItemIdToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MoveTextboxCursor(richTextBoxLogView);
            CopyItemId(richTextBoxLogView);
        }

        private void richTextBoxLogView_MouseUp(object sender, MouseEventArgs e)
        {
            LogMousePosition(new Point(e.X, e.Y));
        }

        private void copyItemIdwithoutChangeKeyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MoveTextboxCursor(richTextBoxLogView);
            CopyItemId(richTextBoxLogView,true);
        }

        private void copyItemIdIdOnlyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MoveTextboxCursor(richTextBoxLogView);
            CopyItemId(richTextBoxLogView, true, true);
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            richTextBoxLogView.Copy();
        }

        private void SelectXMLElementAtCursor(RichTextBox TextBox)
        {
            // Select the XML element at the cursor position
            string sText = TextBox.Text;
            int iStartPos = TextBox.SelectionStart;
            iStartPos = sText.LastIndexOf("<", iStartPos);
            if (sText.Substring(iStartPos + 1, 1) == "/")
            {
                // This is a closing element, so find next...
                iStartPos = sText.LastIndexOf("<", iStartPos - 1);
            }
            int iEndPos = sText.IndexOf("/>", iStartPos);
            if (sText.IndexOf(">", iStartPos) < iEndPos)
                iEndPos = -1;
            if (iEndPos < 0)
            {
                // Need to find closing element
                iEndPos = sText.IndexOf(">", iStartPos);
                if (sText.IndexOf(" ", iStartPos) < iEndPos)
                    iEndPos = sText.IndexOf(" ", iStartPos);
                string sTag = sText.Substring(iStartPos + 1, iEndPos - iStartPos - 1);
                string sClosingTag = "</" + sTag;
                iEndPos = sText.IndexOf(sClosingTag, iStartPos) + sClosingTag.Length;
                if (iEndPos > 0)
                    iEndPos = sText.IndexOf(">", iEndPos);
            }
            else
                iEndPos++;
            if (iEndPos > iStartPos)
            {
                TextBox.SelectionStart = iStartPos;
                TextBox.SelectionLength = iEndPos - iStartPos + 1;
                TextBox.Focus();
            }
        }

        private void selectXMLElementToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MoveTextboxCursor(richTextBoxLogView);
            SelectXMLElementAtCursor(richTextBoxLogView);
        }

        private void checkBoxIndentXML_CheckedChanged(object sender, EventArgs e)
        {
            listViewLogIndex_SelectedIndexChanged(null, null);
        }
    
    }
}
